// Generated by CoffeeScript 1.10.0
(function() {
  var KicadGenerator, fs, log, mkdirp, sprintf;

  fs = require('fs');

  mkdirp = require('mkdirp');

  sprintf = require('sprintf-js').sprintf;

  log = require('./qeda-log');

  KicadGenerator = (function() {
    function KicadGenerator(library) {
      this.library = library;
      this.f = "%." + this.library.pattern.decimals + "f";
    }

    KicadGenerator.prototype.generate = function(name) {
      var dir, element, fd, j, k, len, len1, now, pattern, patternName, patterns, ref, ref1, results, timestamp;
      this.name = name;
      dir = './kicad';
      mkdirp.sync(dir + "/" + this.name + ".pretty");
      patterns = {};
      now = new Date;
      timestamp = sprintf("%02d/%02d/%d %02d:%02d:%02d", now.getDate(), now.getMonth() + 1, now.getYear() + 1900, now.getHours(), now.getMinutes(), now.getSeconds());
      log.start("KiCad library '" + this.name + ".lib'");
      fd = fs.openSync(dir + "/" + this.name + ".lib", 'w');
      fs.writeSync(fd, "EESchema-LIBRARY Version 2.3 Date: " + timestamp + "\n");
      fs.writeSync(fd, '#encoding utf-8\n');
      patterns = {};
      ref = this.library.elements;
      for (j = 0, len = ref.length; j < len; j++) {
        element = ref[j];
        this._generateSymbol(fd, element);
        if (element.pattern != null) {
          patterns[element.pattern.name] = element.pattern;
        }
      }
      fs.writeSync(fd, '# End Library\n');
      fs.closeSync(fd);
      log.ok();
      log.start("KiCad library doc '" + this.name + ".dcm'");
      fd = fs.openSync(dir + "/" + this.name + ".dcm", 'w');
      fs.writeSync(fd, "EESchema-DOCLIB Version 2.0 Date: " + timestamp + "\n#\n");
      ref1 = this.library.elements;
      for (k = 0, len1 = ref1.length; k < len1; k++) {
        element = ref1[k];
        this._generateDoc(fd, element);
      }
      fs.writeSync(fd, '# End Doc Library\n');
      fs.closeSync(fd);
      log.ok();
      results = [];
      for (patternName in patterns) {
        pattern = patterns[patternName];
        log.start("KiCad footprint '" + patternName + ".kicad_mod'");
        fd = fs.openSync(dir + "/" + this.name + ".pretty/" + patternName + ".kicad_mod", 'w');
        this._generatePattern(fd, pattern);
        fs.closeSync(fd);
        results.push(log.ok());
      }
      return results;
    };

    KicadGenerator.prototype._generateDoc = function(fd, element) {
      var empty, field, fields, i, j, k, keys, len, ref;
      fields = ['description', 'keywords', 'datasheet'];
      keys = ['D', 'K', 'F'];
      empty = true;
      for (j = 0, len = fields.length; j < len; j++) {
        field = fields[j];
        if (element[field] != null) {
          empty = false;
          break;
        }
      }
      if (empty) {
        return;
      }
      fs.writeSync(fd, "$CMP " + element.name + "\n");
      for (i = k = 0, ref = fields.length - 1; 0 <= ref ? k <= ref : k >= ref; i = 0 <= ref ? ++k : --k) {
        if (element[fields[i]] != null) {
          fs.writeSync(fd, keys[i] + " " + element[fields[i]] + "\n");
        }
      }
      return fs.writeSync(fd, '$ENDCMP\n#\n');
    };

    KicadGenerator.prototype._generatePattern = function(fd, pattern) {
      var j, len, patObj, ref, shape;
      fs.writeSync(fd, "(module " + pattern.name + " (layer F.Cu)\n");
      if (pattern.type === 'smd') {
        fs.writeSync(fd, "  (attr smd)\n");
      }
      ref = pattern.shapes;
      for (j = 0, len = ref.length; j < len; j++) {
        shape = ref[j];
        patObj = this._patternObj(shape);
        switch (patObj.kind) {
          case 'attribute':
            fs.writeSync(fd, sprintf("  (fp_text %s %s (at " + this.f + " " + this.f + "%s)%s (layer %s)\n", patObj.name, patObj.text, patObj.x, patObj.y, patObj.angle != null ? ' ' + patObj.angle.toString() : '', !patObj.visible ? ' hide' : '', patObj.layer));
            fs.writeSync(fd, sprintf("    (effects (font (size " + this.f + " " + this.f + ") (thickness " + this.f + ")))\n", patObj.fontSize, patObj.fontSize, patObj.lineWidth));
            fs.writeSync(fd, "  )\n");
            break;
          case 'circle':
            fs.writeSync(fd, sprintf("  (fp_circle (center " + this.f + " " + this.f + ") (end " + this.f + " " + this.f + ") (layer %s) (width " + this.f + "))\n", patObj.x, patObj.y, patObj.x, patObj.y + patObj.radius, patObj.layer, patObj.lineWidth));
            break;
          case 'line':
            fs.writeSync(fd, sprintf("  (fp_line (start " + this.f + " " + this.f + ") (end " + this.f + " " + this.f + ") (layer %s) (width " + this.f + "))\n", patObj.x1, patObj.y1, patObj.x2, patObj.y2, patObj.layer, patObj.lineWidth));
            break;
          case 'pad':
            fs.writeSync(fd, sprintf("  (pad %s %s %s (at " + this.f + " " + this.f + ") (size " + this.f + " " + this.f + ") (layers %s)", patObj.name, patObj.type, patObj.shape, patObj.x, patObj.y, patObj.width, patObj.height, patObj.layer));
            if (patObj.drill != null) {
              fs.writeSync(fd, sprintf("\n    (drill " + this.f + ")", patObj.drill));
            }
            if (patObj.mask != null) {
              fs.writeSync(fd, sprintf("\n    (solder_mask_margin " + this.f + ")", patObj.mask));
            }
            if (patObj.paste != null) {
              fs.writeSync(fd, sprintf("\n    (solder_paste_margin " + this.f + ")", patObj.paste));
            }
            fs.writeSync(fd, ")\n");
        }
      }
      return fs.writeSync(fd, ")\n");
    };

    KicadGenerator.prototype._generateSymbol = function(fd, element) {
      var attrObj, hidden, i, j, k, l, len, len1, len2, len3, len4, m, n, nameObj, patternName, pinNameSpace, pointCount, polyPoints, powerSymbol, ref, ref1, ref2, ref3, ref4, ref5, ref6, refObj, shape, showPinNames, showPinNumbers, symObj, symbol;
      ref = element.symbols;
      for (j = 0, len = ref.length; j < len; j++) {
        symbol = ref[j];
        symbol.resize(50, true);
        symbol.invertVertical();
      }
      symbol = element.symbols[0];
      refObj = this._symbolObj(symbol.attributes['refDes']);
      nameObj = this._symbolObj(symbol.attributes['name']);
      fs.writeSync(fd, "#\n# " + element.name + "\n#\n");
      showPinNumbers = ((ref1 = element.schematic) != null ? ref1.showPinNumbers : void 0) ? 'Y' : 'N';
      showPinNames = ((ref2 = element.schematic) != null ? ref2.showPinNames : void 0) ? 'Y' : 'N';
      pinNameSpace = 0;
      ref3 = symbol.shapes;
      for (k = 0, len1 = ref3.length; k < len1; k++) {
        shape = ref3[k];
        if (shape.kind === 'pin') {
          pinNameSpace = Math.round(shape.space);
          break;
        }
      }
      patternName = '';
      if (element.pattern != null) {
        patternName = this.name + ":" + element.pattern.name;
      }
      powerSymbol = 'N';
      if (element.power) {
        powerSymbol = 'P';
      }
      fs.writeSync(fd, "DEF " + element.name + " " + element.refDes + " 0 " + pinNameSpace + " " + showPinNumbers + " " + showPinNames + " " + element.symbols.length + " L " + powerSymbol + "\n");
      fs.writeSync(fd, "F0 \"" + element.refDes + "\" " + refObj.x + " " + refObj.y + " " + refObj.fontSize + " " + refObj.orientation + " " + refObj.visible + " " + refObj.halign + " " + refObj.valign + "NN\n");
      fs.writeSync(fd, "F1 \"" + element.name + "\" " + nameObj.x + " " + nameObj.y + " " + nameObj.fontSize + " " + nameObj.orientation + " " + nameObj.visible + " " + nameObj.halign + " " + nameObj.valign + "NN\n");
      fs.writeSync(fd, "F2 \"" + patternName + "\" 0 0 0 H I C CNN\n");
      if (element.datasheet != null) {
        fs.writeSync(fd, "F3 \"" + element.datasheet + "\" 0 0 0 H I C CNN\n");
      }
      i = 0;
      ref4 = symbol.shapes;
      for (l = 0, len2 = ref4.length; l < len2; l++) {
        shape = ref4[l];
        if ((shape.kind === 'attribute') && (shape.name !== 'refDes') && (shape.name !== 'name')) {
          attrObj = this._symbolObj(shape);
          fs.writeSync(fd, "F" + (4 + i) + " \"" + attrObj.text + "\" " + attrObj.x + " " + attrObj.y + " " + attrObj.fontSize + " " + attrObj.orientation + " V " + attrObj.halign + " " + attrObj.valign + "NN \"" + attrObj.name + "\"\n");
          ++i;
        }
      }
      if (element.aliases != null) {
        fs.writeSync(fd, "ALIAS " + (element.aliases.join(' ')) + "\n");
      }
      if (element.pattern != null) {
        fs.writeSync(fd, "$FPLIST\n");
        fs.writeSync(fd, "  " + element.pattern.name + "\n");
        fs.writeSync(fd, "$ENDFPLIST\n");
      }
      fs.writeSync(fd, "DRAW\n");
      i = 1;
      ref5 = element.symbols;
      for (m = 0, len3 = ref5.length; m < len3; m++) {
        symbol = ref5[m];
        ref6 = symbol.shapes;
        for (n = 0, len4 = ref6.length; n < len4; n++) {
          shape = ref6[n];
          symObj = this._symbolObj(shape);
          switch (symObj.kind) {
            case 'pin':
              fs.writeSync(fd, "X " + symObj.name + " " + symObj.number + " " + symObj.x + " " + symObj.y + " " + symObj.length + " " + symObj.orientation + " " + symObj.fontSize + " " + symObj.fontSize + " " + i + " 1 " + symObj.type + symObj.shape + "\n");
              break;
            case 'rectangle':
              fs.writeSync(fd, "S " + symObj.x1 + " " + symObj.y1 + " " + symObj.x2 + " " + symObj.y2 + " " + i + " 1 " + symObj.lineWidth + " " + symObj.fillStyle + "\n");
              break;
            case 'line':
              fs.writeSync(fd, "P 2 " + i + " 1 " + symObj.lineWidth + " " + symObj.x1 + " " + symObj.y1 + " " + symObj.x2 + " " + symObj.y2 + " N\n");
              break;
            case 'circle':
              fs.writeSync(fd, "C " + symObj.x + " " + symObj.y + " " + symObj.radius + " " + i + " 1 " + symObj.lineWidth + " " + symObj.fillStyle + "\n");
              break;
            case 'arc':
              symObj.start = Math.round(symObj.start * 10);
              symObj.end = Math.round(symObj.end * 10);
              fs.writeSync(fd, "A " + symObj.x + " " + symObj.y + " " + symObj.radius + " " + symObj.start + " " + symObj.end + " " + i + " 1 " + symObj.lineWidth + " " + symObj.fillStyle + "\n");
              break;
            case 'poly':
              pointCount = symObj.points.length / 2;
              polyPoints = symObj.points.reduce(function(p, v) {
                return p + ' ' + v;
              });
              fs.writeSync(fd, "P " + pointCount + " " + i + " 1 " + symObj.lineWidth + " " + polyPoints + " " + symObj.fillStyle + "\n");
              break;
            case 'text':
              hidden = symObj.visible === 'I' ? 1 : 0;
              fs.writeSync(fd, "T " + symObj.angle + " " + symObj.x + " " + symObj.y + " " + symObj.fontSize + " " + hidden + " " + i + " 1 \"" + symObj.text + "\" " + symObj.italic + " " + symObj.bold + " " + symObj.halign + " " + symObj.valign + "\n");
          }
        }
        ++i;
      }
      fs.writeSync(fd, "ENDDRAW\n");
      return fs.writeSync(fd, "ENDDEF\n");
    };

    KicadGenerator.prototype._patternObj = function(shape) {
      var layers, obj, ref, ref1, types;
      obj = shape;
      if (obj.visible == null) {
        obj.visible = true;
      }
      if (obj.shape === 'rectangle') {
        obj.shape = 'rect';
      }
      layers = {
        topCopper: 'F.Cu',
        topMask: 'F.Mask',
        topPaste: 'F.Paste',
        topSilkscreen: 'F.SilkS',
        topAssembly: 'F.Fab',
        topCourtyard: 'F.CrtYd',
        intCopper: '*.Cu',
        bottomCopper: 'B.Cu',
        bottomMask: 'B.Mask',
        bottomPaste: 'B.Paste',
        bottomSilkscreen: 'B.SilkS',
        bottomAssembly: 'B.Fab',
        bottomCourtyard: 'B.CrtYd'
      };
      obj.layer = obj.layer.map((function(_this) {
        return function(v) {
          return layers[v];
        };
      })(this)).join(' ');
      if ((obj.mask != null) && (obj.mask < 0.001)) {
        obj.mask = 0.001;
      }
      if ((obj.paste != null) && (obj.paste > -0.001)) {
        obj.paste = -0.001;
      }
      if (obj.kind === 'attribute') {
        switch (obj.name) {
          case 'refDes':
            obj.name = 'reference';
            obj.text = 'REF**';
            if (obj.fontSize == null) {
              obj.fontSize = this.library.pattern.fontSize.refDes;
            }
            break;
          case 'value':
            if (obj.fontSize == null) {
              obj.fontSize = this.library.pattern.fontSize.value;
            }
            break;
          default:
            obj.name = 'user';
            if (obj.fontSize == null) {
              obj.fontSize = this.library.pattern.fontSize["default"];
            }
        }
      } else if (obj.kind === 'pad') {
        if (obj.type === 'mount') {
          if (obj.shape == null) {
            obj.shape = 'circle';
          }
        }
        if (obj.width == null) {
          obj.width = (ref = obj.drill) != null ? ref : obj.size;
        }
        if (obj.height == null) {
          obj.height = (ref1 = obj.drill) != null ? ref1 : obj.size;
        }
        types = {
          'smd': 'smd',
          'through-hole': 'thru_hole',
          'mounting-hole': 'np_thru_hole'
        };
        obj.type = types[obj.type];
      }
      return obj;
    };

    KicadGenerator.prototype._symbolObj = function(shape) {
      var obj;
      if (shape == null) {
        return;
      }
      obj = shape;
      if (obj.name == null) {
        obj.name = '~';
      }
      if (obj.name === '') {
        obj.name = '~';
      }
      if (obj.visible == null) {
        obj.visible = true;
      }
      obj.visible = obj.visible ? 'V' : 'I';
      obj.halign = (function() {
        switch (obj.halign) {
          case 'center':
            return 'C';
          case 'right':
            return 'R';
          default:
            return 'L';
        }
      })();
      obj.valign = (function() {
        switch (obj.valign) {
          case 'center':
            return 'C';
          case 'bottom':
            return 'B';
          default:
            return 'T';
        }
      })();
      obj.orientation = (function() {
        switch (obj.orientation) {
          case 'left':
            return 'L';
          case 'right':
            return 'R';
          case 'up':
            return 'U';
          case 'down':
            return 'D';
          case 'horizontal':
            return 'H';
          case 'vertical':
            return 'V';
          default:
            return 'H';
        }
      })();
      obj.fontSize = Math.round(obj.fontSize);
      obj.italic = obj.italic ? 'Italic' : 'Normal';
      obj.bold = obj.bold ? 1 : 0;
      obj.type = 'U';
      if (obj.power || obj.ground) {
        obj.type = 'W';
        if (obj.out) {
          obj.type = 'w';
        }
      } else {
        if (obj.bidir || (obj["in"] && obj.out)) {
          obj.type = 'B';
        } else if (obj["in"]) {
          obj.type = 'I';
        } else if (obj.out) {
          obj.type = 'O';
        } else if (obj.passive) {
          obj.type = 'P';
        } else if (obj.nc) {
          obj.type = 'N';
        } else if (obj.z) {
          obj.type = 'T';
        } else {
          obj.type = 'U';
        }
      }
      obj.shape = '';
      if (obj.invisible) {
        obj.shape += 'N';
      }
      if (obj.inverted) {
        obj.shape += 'I';
      }
      if (obj.shape !== '') {
        obj.shape = ' ' + obj.shape;
      }
      switch (obj.fill) {
        case 'foreground':
          obj.fillStyle = 'f';
          break;
        case 'background':
          obj.fillStyle = 'F';
          break;
        default:
          obj.fillStyle = 'N';
      }
      return obj;
    };

    return KicadGenerator;

  })();

  module.exports = KicadGenerator;

}).call(this);
