// Generated by CoffeeScript 1.10.0
(function() {
  var QedaPattern, crc, parse,
    hasProp = {}.hasOwnProperty;

  crc = require('crc');

  parse = require('svg-path-parser');

  QedaPattern = (function() {
    function QedaPattern(element) {
      this.settings = element.library.pattern;
      this.crc32 = this._calcCrc(element.housing);
      this.shapes = [];
      this.currentLayer = ['topCopper'];
      this.currentLineWidth = 0;
      this.type = 'smd';
      this.attributes = {};
      this.pads = {};
      this.x = 0;
      this.y = 0;
      this.cx = 0;
      this.cy = 0;
    }

    QedaPattern.prototype.attribute = function(name, attribute) {
      attribute.name = name;
      attribute.x = this.cx + attribute.x;
      attribute.y = this.cy + attribute.y;
      this.attributes[name] = this._addShape('attribute', attribute);
      return this;
    };

    QedaPattern.prototype.center = function(x, y) {
      this.cx = x;
      this.cy = y;
      return this;
    };

    QedaPattern.prototype.circle = function(x, y, radius) {
      this._addShape('circle', {
        x: this.cx + x,
        y: this.cy + y,
        radius: radius
      });
      return this;
    };

    QedaPattern.prototype.extremePads = function() {
      var firstPad, lastPad, numbers;
      numbers = Object.keys(this.pads);
      firstPad = this.pads[numbers[0]];
      lastPad = this.pads[numbers[numbers.length - 1]];
      return [firstPad, lastPad];
    };

    QedaPattern.prototype.isEqualTo = function(pattern) {
      return this.crc32 === pattern.crc32;
    };

    QedaPattern.prototype.layer = function(layer) {
      if (!Array.isArray(layer)) {
        layer = [layer];
      }
      this.currentLayer = layer;
      return this;
    };

    QedaPattern.prototype.line = function(x1, y1, x2, y2) {
      if ((x1 !== x2) || (y1 !== y2)) {
        this._addShape('line', {
          x1: this.cx + x1,
          y1: this.cy + y1,
          x2: this.cx + x2,
          y2: this.cy + y2
        });
      }
      return this;
    };

    QedaPattern.prototype.lineTo = function(x, y) {
      this.line(this.x, this.y, x, y);
      return this.moveTo(x, y);
    };

    QedaPattern.prototype.lineWidth = function(lineWidth) {
      this._setLineWidth(lineWidth);
      return this;
    };

    QedaPattern.prototype.moveTo = function(x, y) {
      this.x = x;
      this.y = y;
      return this;
    };

    QedaPattern.prototype.pad = function(name, pad) {
      pad.name = name;
      pad.x = this.cx + pad.x;
      pad.y = this.cy + pad.y;
      this.pads[name] = this._addPad(pad);
      return this;
    };

    QedaPattern.prototype.parseArray = function(input) {
      var result;
      if (input == null) {
        return [0];
      }
      result = input.toString().replace(/\s+/g, '').split(',').map((function(_this) {
        return function(v) {
          return parseFloat(v);
        };
      })(this));
      return result;
    };

    QedaPattern.prototype.parsePosition = function(value) {
      var i, j, l, len, len1, points, values, x, y;
      values = value.replace(/\s+/g, '').split(',').map((function(_this) {
        return function(v) {
          return parseFloat(v);
        };
      })(this));
      points = [];
      for (j = 0, len = values.length; j < len; j += 2) {
        x = values[j];
        points.push({
          x: x
        });
      }
      values.shift();
      for (i = l = 0, len1 = values.length; l < len1; i = l += 2) {
        y = values[i];
        points[i / 2].y = y;
      }
      return points;
    };

    QedaPattern.prototype.path = function(d) {
      var j, len, p, parts;
      parts = parse(d);
      for (j = 0, len = parts.length; j < len; j++) {
        p = parts[j];
        switch (p.code) {
          case 'l':
            this.lineTo(this.x + p.x, this.y + p.y);
            break;
          case 'L':
            this.lineTo(p.x, p.y);
            break;
          case 'm':
            this.moveTo(this.x + p.x, this.y + p.y);
            break;
          case 'M':
            this.moveTo(p.x, p.y);
        }
      }
      return this;
    };

    QedaPattern.prototype.polarityMark = function(x, y, position) {
      var d, oldLineWidth, r;
      if (position == null) {
        position = 'left';
      }
      d = 0.5;
      switch (position) {
        case 'left':
          x -= d / 2;
          break;
        case 'right':
          x += d / 2;
          break;
        case 'top':
          y -= d / 2;
          break;
        case 'bottom':
          y += d / 2;
          break;
        case 'topLeft':
          x -= d / 2;
          y -= d / 2;
          break;
        case 'topRight':
          x += d / 2;
          y -= d / 2;
          break;
        case 'bottomLeft':
          x -= d / 2;
          y += d / 2;
          break;
        case 'bottomRight':
          x += d / 2;
          y += d / 2;
      }
      switch (this.settings.polarityMark) {
        case 'dot':
          r = d / 2;
          oldLineWidth = this.currentLineWidth;
          this.lineWidth(r);
          this.circle(x, y, r / 2);
          return this.lineWidth(oldLineWidth);
      }
    };

    QedaPattern.prototype.rectangle = function(x1, y1, x2, y2) {
      this.save();
      this.moveTo(x1, y1);
      this.lineTo(x2, y1);
      this.lineTo(x2, y2);
      this.lineTo(x1, y2);
      this.lineTo(x1, y1);
      return this.restore();
    };

    QedaPattern.prototype.rectangleTo = function(x, y) {
      this.rectangle(this.x, this.y, x, y);
      return this.moveTo(x, y);
    };

    QedaPattern.prototype.restore = function() {
      var ref;
      if (this._origin == null) {
        return;
      }
      ref = this._origin.pop(), this.x = ref[0], this.y = ref[1];
      return this;
    };

    QedaPattern.prototype.save = function() {
      if (this._origin == null) {
        this._origin = [];
      }
      this._origin.push([this.x, this.y]);
      return this;
    };

    QedaPattern.prototype.silkLine = function(x1, y1, x2, y2) {
      var line, lines, n, newlines, pad, parts, ref;
      if ((x1 !== x2) || (y1 !== y2)) {
        lines = [
          {
            x1: this.cx + x1,
            y1: this.cy + y1,
            x2: this.cx + x2,
            y2: this.cy + y2
          }
        ];
        ref = this.pads;
        for (n in ref) {
          pad = ref[n];
          newlines = [];
          while (lines.length) {
            line = lines.shift();
            parts = this._divideLine(line, pad, this.settings.clearance.padToSilk + this.settings.lineWidth.silkscreen / 2);
            newlines = newlines.concat(parts);
          }
          lines = newlines;
        }
        lines.map((function(_this) {
          return function(v) {
            return _this._addShape('line', {
              x1: v.x1,
              y1: v.y1,
              x2: v.x2,
              y2: v.y2
            });
          };
        })(this));
      }
      return this;
    };

    QedaPattern.prototype.silkRectangle = function(x1, y1, x2, y2) {
      this.silkLine(x1, y1, x2, y1);
      this.silkLine(x2, y1, x2, y2);
      this.silkLine(x2, y2, x1, y2);
      return this.silkLine(x1, y2, x1, y1);
    };

    QedaPattern.prototype._addPad = function(pad) {
      if (pad.type !== 'smd') {
        this.type = 'through-hole';
      }
      return this._addShape('pad', pad);
    };

    QedaPattern.prototype._addShape = function(kind, shape) {
      var obj, prop;
      obj = {
        kind: kind
      };
      for (prop in shape) {
        if (!hasProp.call(shape, prop)) continue;
        obj[prop] = shape[prop];
      }
      if (obj.layer == null) {
        obj.layer = this.currentLayer;
      }
      if (obj.lineWidth == null) {
        obj.lineWidth = this.currentLineWidth;
      }
      this.shapes.push(obj);
      return obj;
    };

    QedaPattern.prototype._calcCrc = function(housing) {
      var exclude, j, key, len, ref, sum;
      sum = 0;
      exclude = ['suffix'];
      ref = Object.keys(housing);
      for (j = 0, len = ref.length; j < len; j++) {
        key = ref[j];
        if (exclude.indexOf(key) !== -1) {
          continue;
        }
        sum = crc.crc32(key + "=" + housing[key], sum);
      }
      return sum;
    };

    QedaPattern.prototype._divideLine = function(line, pad, space) {
      var bottom, intersect, k, left, lines, px1, px2, py1, py2, result, right, top, x, y;
      px1 = pad.x - pad.width / 2 - space;
      px2 = pad.x + pad.width / 2 + space;
      py1 = pad.y - pad.height / 2 - space;
      py2 = pad.y + pad.height / 2 + space;
      lines = [];
      intersect = false;
      if ((line.x1 >= px1) && (line.x1 <= px2) && (line.x2 >= px1) && (line.x2 <= px2) && (line.y1 >= py1) && (line.y1 <= py2) && (line.y2 >= py1) && (line.y2 <= py2)) {
        return [];
      }
      if (line.x1 < line.x2) {
        left = {
          x: line.x1,
          y: line.y1
        };
        right = {
          x: line.x2,
          y: line.y2
        };
      } else {
        left = {
          x: line.x2,
          y: line.y2
        };
        right = {
          x: line.x1,
          y: line.y1
        };
      }
      if (left.x < px1 && right.x > px1) {
        k = (right.y - left.y) / (right.x - left.x);
        y = left.y + k * (px1 - left.x);
        if (y > py1 && y < py2) {
          intersect = true;
          lines.push({
            x1: left.x,
            y1: left.y,
            x2: px1,
            y2: y
          });
        }
      }
      if (left.x < px2 && right.x > px2) {
        k = (right.y - left.y) / (right.x - left.x);
        y = left.y + k * (px2 - left.x);
        if (y > py1 && y < py2) {
          intersect = true;
          lines.push({
            x1: px2,
            y1: y,
            x2: right.x,
            y2: right.y
          });
        }
      }
      if (line.y1 < line.y2) {
        top = {
          x: line.x1,
          y: line.y1
        };
        bottom = {
          x: line.x2,
          y: line.y2
        };
      } else {
        top = {
          x: line.x2,
          y: line.y2
        };
        bottom = {
          x: line.x1,
          y: line.y1
        };
      }
      if (top.y < py1 && bottom.y > py1) {
        k = (bottom.x - top.x) / (bottom.y - top.y);
        x = top.x + k * (py1 - top.y);
        if (x > px1 && x < px2) {
          intersect = true;
          lines.push({
            x1: top.x,
            y1: top.y,
            x2: x,
            y2: py1
          });
        }
      }
      if (top.y < py2 && bottom.y > py2) {
        k = (bottom.x - top.x) / (bottom.y - top.y);
        x = top.x + k * (py2 - top.y);
        if (x > px1 && x < px2) {
          intersect = true;
          lines.push({
            x1: x,
            y1: py2,
            x2: bottom.x,
            y2: bottom.y
          });
        }
      }
      if (!intersect) {
        lines.push(line);
      }
      result = lines.filter((function(_this) {
        return function(v) {
          return _this._lineLength(v) > _this.settings.lineWidth.silkscreen;
        };
      })(this));
      return result;
    };

    QedaPattern.prototype._lineLength = function(line) {
      var dx, dy;
      dx = line.x2 - line.x1;
      dy = line.y2 - line.y1;
      return Math.sqrt(dx * dx + dy * dy);
    };

    QedaPattern.prototype._mergeObjects = function(dest, src) {
      var k, results, v;
      results = [];
      for (k in src) {
        v = src[k];
        if (typeof v === 'object' && dest.hasOwnProperty(k)) {
          results.push(this.mergeObjects(dest[k], v));
        } else {
          results.push(dest[k] = v);
        }
      }
      return results;
    };

    QedaPattern.prototype._setLineWidth = function(lineWidth) {
      return this.currentLineWidth = lineWidth;
    };

    return QedaPattern;

  })();

  module.exports = QedaPattern;

}).call(this);
